@@ -1,11 +1,13 @@
-__version__ = "0.3.6"
+__version__ = "0.4.1"
+__package__ = "oneai"
 
 from typing_extensions import Final
 import oneai.skills as skills
 from oneai.classes import *
 from oneai.pipeline import Pipeline
+import oneai.clustering as clustering
 
-URL: Final[str] = "https://api.oneai.com/api/v0/pipeline"
+URL: Final[str] = "https://api.oneai.com"
 """
 Base URL for the pipeline API. Only change if you know what you're doing.
 """
