@@ -1,9 +1,9 @@
 from base64 import b64encode
 from dataclasses import dataclass, field
-from io import BytesIO
 import json
 import os
-from typing import BinaryIO, Callable, Iterable, List, Literal, TextIO, Type, Union
+from typing import Any, Callable, Dict, Iterable, List, Type, Union
+from warnings import warn
 
 import aiohttp
 
@@ -37,9 +37,8 @@ class Skill:
         The attribute name of the Skill's output in the Output object.
     `output_attr1: str`
         Only for Skills with 2 outputs (text / labels)
-    `run_custom: (Output, aiohttp.ClientSession) -> str | list[Label] | Output`
+    `run_custom: (Input, aiohttp.ClientSession) -> str | list[Label] | Output`
         A custom function that will be called locally instead of passing the Skill to the API.
-        The `Output` object hold the input text (`Output.text`), and output of the previous Skills in the pipeline.
         Can return a string with generated text (when `is_generator=True`), a list of `Label`s (when `is_generator=False`) or an `Output` object for more complex outputs.
         Can use the `aiohttp.ClientSession` to make HTTP requests.
     """
@@ -51,14 +50,18 @@ class Skill:
     label_type: str = ""
     output_attr: str = ""
     output_attr1: str = field(default="", repr=False)
-    run_custom: Callable[['Output', aiohttp.ClientSession], Union[str, 'Labels', 'Output']] = None
+    run_custom: Callable[
+        ["Input", aiohttp.ClientSession], Union[str, "Labels", "Output"]
+    ] = None
 
     def asdict(self) -> dict:
         return {
             "skill": self.api_name,
             "params": {
-                p: self.__getattribute__(p) for p in self._skill_params if self.__getattribute__(p)
-            }
+                p: self.__getattribute__(p)
+                for p in self._skill_params
+                if self.__getattribute__(p)
+            },
         }
 
 
@@ -69,7 +72,9 @@ def skillclass(
     is_generator: bool = False,
     output_attr: str = "",
     output_attr1: str = "",
-    run_custom: Callable[['Output', aiohttp.ClientSession], Union[str, 'Labels', 'Output']] = None 
+    run_custom: Callable[
+        ["Input", aiohttp.ClientSession], Union[str, "Labels", "Output"]
+    ] = None,
 ):
     """
     A decorator for defining a Language Skill class. Decorate subclasses of `Skill` with this decorator to provide default values for instance attributes.
@@ -77,8 +82,9 @@ def skillclass(
 
     def wrap(cls) -> cls:
         if not issubclass(cls, Skill):
-            print(
-                f"warning: class {cls.__name__} decorated with @skillclass does not inherit Skill"
+            warn(
+                f"warning: class {cls.__name__} decorated with @skillclass does not inherit Skill",
+                stacklevel=2,
             )
 
         def __init__(self, *args, **kwargs):
@@ -113,6 +119,12 @@ class Input:
 
     `type: str`
         A type hint for the API, suggesting which models to use when processing the input.
+    `content_type: str`
+        The content type of the input.
+    `encoding: str`
+        The encoding of the input.
+    `metadata: dict`
+        Optional metadata to be associated with the input in clustering collections. Not implemented by default. 
 
     ## Methods
 
@@ -122,8 +134,10 @@ class Input:
         A class method. Parse a string into an instance of the Input class. Not implemented by default.
     """
 
-    def __init__(self, type: str):
+    def __init__(self, type: str, content_type: str = None, encoding: str = None):
         self.type = type
+        self.content_type = content_type
+        self.encoding = encoding
 
     @classmethod
     def parse(cls, text: str) -> "Input":
@@ -141,9 +155,10 @@ class Input:
         """
         raise NotImplementedError()
 
-    def get_text(self) -> str:
+    @property
+    def raw(self) -> str:
         """
-        Returns the input as a string (used by `repr`). Not implemented by default.
+        Returns the input as a raw string. Not implemented by default.
 
         ## Returns
 
@@ -151,8 +166,16 @@ class Input:
         """
         raise NotImplementedError()
 
-    def __repr__(self) -> str:
-        return self.get_text()
+    @property
+    def metadata(self) -> Dict[str, Any]:
+        """
+        Returns the metadata of the input. Not implemented by default.
+
+        ## Returns
+
+        A `str`-key dict of input-associated metadata.
+        """
+        raise NotImplementedError()
 
 
 class Document(Input):
@@ -171,10 +194,11 @@ class Document(Input):
     `parse(text) -> Document`
         A class method. Parse a string into a `Document` instance.
     """
-    type = 'article'
+
+    type = "article"
 
     def __init__(self, text: str):
-        self.text = text
+        self.raw = text
 
     @classmethod
     def parse(cls, text: str) -> "Document":
@@ -192,14 +216,8 @@ class Document(Input):
         """
         return cls(text)
 
-    def get_text(self) -> str:
-        """
-        Returns the document as a string.
-
-        ## Returns
-
-        `str` representation of this `Documentation` instance.
-        """
+    @property
+    def raw(self) -> str:
         return self.text
 
 
@@ -229,12 +247,13 @@ class Conversation(Input):
         A class method. Parse a string with a structued conversation format or a conversation JSON string into a `Conversation` instance.
     """
 
-    type = 'conversation'
+    type = "conversation"
 
     def __init__(self, utterances: List[Utterance] = []):
         self.utterances = utterances
 
-    def get_text(self) -> str:
+    @property
+    def raw(self) -> str:
         """
         Returns the conversation as a JSON string.
 
@@ -244,10 +263,13 @@ class Conversation(Input):
         """
         return json.dumps(self.utterances, default=lambda o: o.__dict__)
 
+    def __getitem__(self, index: int) -> Utterance:
+        return self.utterances[index]
+
     @classmethod
     def parse(cls, text: str) -> "Conversation":
         """
-        A class method. Parse a string with a structued conversation format or a conversation JSON string into a `Conversation` instance.
+        A class method. Parse a string with a structured conversation format or a conversation JSON string into a `Conversation` instance.
 
         ## Parameters
 
@@ -270,75 +292,120 @@ class Conversation(Input):
 
             return parse_conversation(text)
 
-    def __str__(self) -> str:
+    def __repr__(self) -> str:
         return f"oneai.Conversation{repr(self.utterances)}"
 
 
-class Audio(Input):
+class File(Input):
     """
-    Represents audio inputs.
+    Represents file inputs. Supported file extensions:
+    * .txt (text files)
+    * .wav, .mp3 (transcribe)
+    * .srt (captions)
+    * .jpg (OCR)
+    * .json (One AI conversation JSON format)
 
     ## Attributes
 
     `data: str`
-        A base64-encoded string of the audio data.
+        Encoded file data.
     `type: str`
-        An input-type hint for the API, either `Conversation.type` or `Document.type`.
+        An input-type hint for the API, either `Conversation` or `Document`.
+    `content_type: str`
+        The content type of the file.
+    `encoding: str`
+        The encoding of the file.
 
     ## Methods
 
     `get_text() -> str`
-        Returns the encoded audio data.
+        Returns the encoded file data.
     """
 
-    def __init__(
-        self,
-        file: Union[str, BinaryIO, bytes],
-        extension: str=None,
-        type: str=Conversation.type
-    ):
+    def __init__(self, file_path: str, type: Union[str, Type[Input]] = None):
         """
-        Creates a new `Audio` input instance
+        Creates a new `File` input instance
 
         ## Parameters
 
-        `file: Union[str, BinaryIO]`
-            The audio file to encode. Either a `str` file name, `BinaryIO` object or `bytes` data.
-        `extension: str` (optional)
-            The file extension of the audio file. If not provided, it will be guessed from the file name.
-            Only `.wav` files are supported at the time.
+        `file_path: str`
+            The path of the file to encode. Supported file extensions: [.txt, .wav, .mp3, .srt, .jpg/.jpeg, .json]
         `type: str` (optional)
-            The input-type hint for the API, either `Conversation.type` or `Document.type`.
+            The input-type hint for the API, either `Conversation` or `Document`.
         """
-        super().__init__(type)
+        if type is None or isinstance(type, str):
+            super().__init__(type)
+        elif issubclass(type, Input):
+            super().__init__(type.type)
+        else:
+            raise ValueError(f"Invalid input type {type}")
 
-        if isinstance(file, str):
-            _, ext = os.path.splitext(file)
-            if extension and extension not in ext:
-                raise InputError(message='file extension does not match', details=f'file name: {file}, extension: {extension}')
-            elif ext in ['.wav']:
-                file = open(file, 'rb').read()
-            else:
-                raise InputError(message=f'unsupported file extension {ext}', details='only .wav files are supported at the time')
-        elif isinstance(file, BytesIO):
-            file = file.read()
-        
-        self.content_type = 'audio/wav'
-        
-        self.data = b64encode(file).decode('utf-8')
-        self.encoding = 'base64'
-    
-    def get_text(self) -> str:
+        utf8, b64 = "utf8", "b64"
+        _, ext = os.path.splitext(file_path)
+        if ext == ".json":
+            self.content_type = "application/json"
+            self.encoding = utf8
+        elif ext == ".txt":
+            self.content_type = "text/plain"
+            self.encoding = utf8
+        elif ext == ".srt":
+            self.data = Conversation.parse(open(file_path).read()).raw
+            return
+        elif ext in [".jpg", ".jpeg"]:
+            self.content_type = "image/jpeg"
+            self.encoding = b64
+        elif ext == ".wav":
+            self.content_type = "audio/wav"
+            self.encoding = b64
+        elif ext == ".mp3":
+            self.content_type = "audio/mp3"
+            self.encoding = b64
+        elif ext == ".html":
+            self.content_type = "text/html"
+            self.encoding = utf8
+        else:
+            raise InputError(
+                message=f"unsupported file extension {ext}",
+                details="see supported files in docs",
+            )
+
+        if self.encoding == utf8:
+            self.data = open(file_path).read()
+        else:
+            self.data = b64encode(file_path).decode("utf-8")
+
+    @property
+    def raw(self) -> str:
         """
-        Returns the encoded audio data.
+        Returns the encoded file data.
 
         ## Returns
 
-        `str` representation of this `Audio` instance.
+        `str` representation of this `File` instance.
         """
 
         return self.data
 
+    @classmethod
+    def parse(cls, text: str) -> Input:
+        """
+        A class method. Parse a string into an `Input` instance.
+
+        ## Parameters
+
+        `text: str`
+            The text to parse.
+
+        ## Returns
+
+        The `Input` instance produced from `text`.
+        """
+        try:
+            return Conversation.parse(text)
+        except:
+            return text
+
+
 @dataclass
 class Span:
     start: int
@@ -347,13 +414,20 @@ class Span:
     text: str = None
 
     @classmethod
-    def from_json(cls, objects: List[dict], text: str) -> "List[Span]":
-        return [] if not objects else [cls(
-            start=object.get("start", None),
-            end=object.get("end", None),
-            section=object.get("section", None),
-            text=text
-        ) for object in objects]
+    def from_dict(cls, objects: List[dict], text: str) -> "List[Span]":
+        return (
+            []
+            if not objects
+            else [
+                cls(
+                    start=object.get("start", None),
+                    end=object.get("end", None),
+                    section=object.get("section", None),
+                    text=text,
+                )
+                for object in objects
+            ]
+        )
 
 
 @dataclass
@@ -380,22 +454,37 @@ class Label:
     """
 
     type: str = ""
+    skill: str = ""
     name: str = ""
-    span: List[int] = field(default_factory=lambda: [0, 0])
+    _span: List[int] = field(default_factory=lambda: [0, 0], repr=False)
     output_spans: List[int] = field(default_factory=list)
     input_spans: List[int] = field(default_factory=list)
     span_text: str = ""
     value: str = ""
     data: dict = field(default_factory=dict)
 
+    @property
+    def span(self) -> Span:
+        warn(
+            "`Label.span` is deprecated, use `Label.output_spans` instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return self._span
+
     @classmethod
-    def from_json(cls, object: dict) -> "Label":
+    def from_dict(cls, object: dict) -> "Label":
         return cls(
             type=object.pop("type", ""),
+            skill=object.pop("skill", ""),
             name=object.pop("name", ""),
-            output_spans=Span.from_json(object.pop("output_spans", []), object.get('span_text', None)),
-            input_spans=Span.from_json(object.pop("input_spans", []), object.get('span_text', None)),
-            span=object.pop("span", [0, 0]),
+            output_spans=Span.from_dict(
+                object.pop("output_spans", []), object.get("span_text", None)
+            ),
+            input_spans=Span.from_dict(
+                object.pop("input_spans", []), object.get("span_text", None)
+            ),
+            _span=object.pop("span", [0, 0]),
             span_text=object.pop("span_text", ""),
             value=object.pop("value", ""),
             data=object.pop("data", {}),
@@ -404,7 +493,7 @@ class Label:
     def __repr__(self) -> str:
         return (
             "oneai.Label("
-            + ", ".join(f"{k}={repr(v)}" for k, v in self.__dict__.items() if v)
+            + ", ".join(f"{k}={repr(v)}" for k, v in self.__dict__.items() if v and not k.startswith("_"))
             + ")"
         )
 
@@ -413,27 +502,38 @@ class Labels(List[Label]):
     """
     Wrapper object for a list of `Label` objects. Provides convenience methods to query labels by attribute.
 
-    ## Methods
+    ## Properties
 
-    `values() -> list[str]`
-        Returns a list of all values of the labels.
-    `names() -> list[str]`
-        Returns a list of all names of the labels.
-    `spans() -> list[list[int]]`
-        Returns a list of all spans of the labels.
-    `values() -> list[str]`
-        Returns a list of all span texts of the labels.
+    `values: list[str]`
+        A list of all values of the labels.
+    `names: list[str]`
+        A list of all names of the labels.
+    `input_spans: list[list[Span]]`
+        A list of all input spans of the labels.
+    `output_spans: list[list[Span]]`
+        A list of all output spans of the labels.
+    `span_texts: list[str]`
+        A list of all span texts of the labels.
     """
-    def values(self):
+
+    @property
+    def values(self) -> List[Any]:
         return [l.value for l in self]
 
-    def names(self):
+    @property
+    def names(self) -> List[str]:
         return [l.name for l in self]
 
-    def spans(self):
-        return [l.span for l in self]
+    @property
+    def input_spans(self) -> List[List[Span]]:
+        return [l.input_spans for l in self]
 
-    def span_texts(self):
+    @property
+    def output_spans(self) -> List[List[Span]]:
+        return [l.output_spans for l in self]
+
+    @property
+    def span_texts(self) -> List[str]:
         return [l.span_text for l in self]
 
 
@@ -456,7 +556,9 @@ class Output(Input):
     """
 
     text: Union[Input, str] = ""
-    skills: List[Skill] = field(default_factory=list, repr=False)  # not a dict since Skills are currently mutable & thus unhashable
+    skills: List[Skill] = field(
+        default_factory=list, repr=False
+    )  # not a dict since Skills are currently mutable & thus unhashable
     data: List[Union[Labels, "Output"]] = field(default_factory=list, repr=False)
 
     def __getitem__(self, name: str) -> Union[Labels, "Output"]:
@@ -464,8 +566,8 @@ class Output(Input):
 
     def __getattr__(self, name: str) -> Union[Labels, "Output"]:
         #####TEMP#HACK########
-        if name == "business_entities" and hasattr(self, "labs"):
-            return self.__getattr__("labs").business_entities
+        if name in ["business_entities", "pricing"] and hasattr(self, "labs"):
+            return getattr(self.__getattr__("labs"), name)
         ######################
         for i, skill in enumerate(self.skills):
             if (
@@ -481,15 +583,16 @@ class Output(Input):
             skill.output_attr or skill.api_name for skill in self.skills
         ]
 
-    def get_text(self) -> str:
-        return self.text if isinstance(self.text, str) else repr(self.text)
+    @property
+    def raw(self) -> str:
+        return self.text if isinstance(self.text, str) else self.raw
 
-    def add(self, skill: Skill, data: Union['Output', Labels]):
+    def add(self, skill: Skill, data: Union["Output", Labels]):
         """
         Add data generated by a `Skill` to the output. Data can be a nested `Output` instance or a list of `Label`s.
 
         ## Parameters
-        
+
         `skill: Skill`
             The Skill which generated the data.
         `data: Output | list[Label]`
