@@ -1,4 +1,6 @@
 import asyncio, concurrent.futures
+import sys
+import os
 import oneai
 
 from typing import Awaitable, Dict, Iterable, List, Union
@@ -65,7 +67,7 @@ class Pipeline:
                 start = i + 1
                 self._segments.append(CustomSegment(skill))
         if i + 1 - start:
-            self._segments.append(APISegment(self.steps[start:i + 1]))
+            self._segments.append(APISegment(self.steps[start : i + 1]))
 
     def run(
         self, input: Union[str, Input, Iterable[Union[str, Input]]], api_key: str = None
@@ -184,11 +186,19 @@ class Pipeline:
 
 # for jupyter environment, to avoid "asyncio.run() cannot be called from a running event loop"
 pool = concurrent.futures.ThreadPoolExecutor()
+is_36 = sys.version_info[:2] == (3, 6)
+
+if os.name == "nt" and not is_36:  # Windows
+    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
 
 
 def _async_run_nested(coru):
-    try:
-        asyncio.get_running_loop()
-        return pool.submit(asyncio.run, coru).result()
-    except RuntimeError:
-        return asyncio.run(coru)
+    if is_36:
+        loop = asyncio.get_event_loop()
+        return loop.run_until_complete(coru)
+    else:
+        try:
+            asyncio.get_running_loop()
+            return pool.submit(asyncio.run, coru).result()
+        except RuntimeError:
+            return asyncio.run(coru)
