@@ -36,6 +36,14 @@ class TranscriptionEnhancer(Skill):
     oneai.Output(text='ENHANCED TRANSCRIPTION', replacements=[...])
     """
 
+    def __new__(cls, *args, **kwargs):
+        warn(
+            "TranscriptionEnhancer Skill is deprecated- use `Proofread` instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return super().__new__(cls, *args, **kwargs)
+
 
 @skillclass(
     api_name="enhance",
@@ -155,8 +163,13 @@ class Entities(Skill):
     [oneai.Label(type=entity, name=PERSON, span=[0, 3], value=Jim), oneai.Label(type=entity, name=ORG, span=[21, 31], value=Acme Corp.)]
     """
 
-    def __post_init__(self):
-        warn("Entities Skill is deprecated- use either `Names` or `Numbers` instead")
+    def __new__(cls, *args, **kwargs):
+        warn(
+            "Entities Skill is deprecated- use either `Names` or `Numbers` instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return super().__new__(cls, *args, **kwargs)
 
 
 @skillclass(api_name="keywords", label_type="keyword")
@@ -253,8 +266,6 @@ class HTMLExtractArticle(Skill):
     """
     Extracts the main text content of an HTML page. Accepts URLs or HTML strings
 
-    Using URLs is deprecated, send HTML strings or extract HTML content manually instead (try [BeautifulSoup4](https://pypi.org/project/beautifulsoup4/))
-
     ## Output
 
     Main text content from the HTML page
@@ -272,14 +283,20 @@ class HTMLExtractArticle(Skill):
     // Bringing human-level language AI to everyday life, one developer a...
     """
 
+    def __new__(cls, *args, **kwargs):
+        warn(
+            "HTMLExtractArticle Skill is deprecated- use `HtmlToArticle` instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return super().__new__(cls, *args, **kwargs)
 
-@skillclass(api_name="html-extract-text", is_generator=True, output_attr="html_text")
+
+@skillclass(api_name="html-extract-text", is_generator=True, output_attr="html_text", run_custom=scraping.extract_text)
 class HTMLExtractText(Skill):
     """
     Extracts all text content of an HTML page (including text from control elements). Accepts URLs or HTML strings. Use HTMLExtractArticle to extract the main content only
 
-    Using URLs is deprecated, send HTML strings or extract HTML content manually instead (try [BeautifulSoup4](https://pypi.org/project/beautifulsoup4/))
-
     ## Output
 
     Text content from the HTML page
@@ -295,6 +312,63 @@ class HTMLExtractText(Skill):
     One AI - About usSkip to main contentAPIStudioSkillsIndustriesPricingResearchAboutAbout UsLeadership...
     """
 
+    def __new__(cls, *args, **kwargs):
+        warn(
+            "HTMLExtractText Skill is deprecated- use `HtmlAllText` instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return super().__new__(cls, *args, **kwargs)
+
+
+@skillclass(
+    api_name="extract-html",
+    is_generator=True,
+    output_attr="html_article",
+    run_custom=scraping.extract_article,
+)
+class HtmlToArticle(Skill):
+    """
+    Extracts the main text content of an HTML page. Accepts URLs or HTML strings
+
+    ## Output
+
+    Main text content from the HTML page
+
+    ## Example
+
+    >>> resp = requests.get('https://oneai.com/about-us/')
+    >>> pipeline = oneai.Pipeline(steps=[
+    ...     oneai.skills.HtmlToArticle()
+    ... ])
+    >>> output = pipeline.run(resp.text)
+    >>> output.html_article.text[:100] + '...'
+    One AI - About us
+    ABOUT ONE AI
+    // Bringing human-level language AI to everyday life, one developer a...
+    """
+
+
+@skillclass(api_name="html-extract-text", is_generator=True, output_attr="html_text", run_custom=scraping.extract_text)
+class HtmlAllText(Skill):
+    """
+    Extracts all text content of an HTML page (including text from control elements). Accepts URLs or HTML strings. Use HTMLExtractArticle to extract the main content only
+
+    ## Output
+
+    Text content from the HTML page
+
+    ## Example
+
+    >>> resp = requests.get('https://oneai.com/about-us/')
+    >>> pipeline = oneai.Pipeline(steps=[
+    ...     oneai.skills.HtmlAllText()
+    ... ])
+    >>> output = pipeline.run(resp.text)
+    >>> output.html_text.text[:100] + '...'
+    One AI - About usSkip to main contentAPIStudioSkillsIndustriesPricingResearchAboutAbout UsLeadership...
+    """
+
 
 @skillclass(
     api_name="business-entities",
@@ -307,7 +381,7 @@ class BusinessEntities(Skill):
     """
     ### 'Labs' Skill- this Skill is still in beta and is may produce incorrect results in some cases
 
-    deprecated- use either `Pricings` instead.
+    deprecated- use `Pricing` instead.
     Detects business entities (quantities, pricings) in the input
 
     ## Output Attributes
@@ -325,8 +399,13 @@ class BusinessEntities(Skill):
     [oneai.Label(type=business-entity, name=pricing, data={'amount': 10.0, 'currency': 'USD', 'unit': 'barrel'}, span=[9, 26], value=10 USD per barrel)]
     """
 
-    def __post_init__(self):
-        warn("BusinessEntities Skill is deprecated- use either `Pricings` instead")
+    def __new__(cls, *args, **kwargs):
+        warn(
+            "BusinessEntities Skill is deprecated- use `Pricing` instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return super().__new__(cls, *args, **kwargs)
 
 
 @skillclass(
@@ -392,26 +471,26 @@ class Anonymize(Skill):
     is_generator=True,
     label_type="business-entity",
     output_attr="labs",
-    output_attr1="pricings",
+    output_attr1="pricing",
 )
-class Pricings(Skill):
+class Pricing(Skill):
     """
     ### 'Labs' Skill- this Skill is still in beta and is may produce incorrect results in some cases
 
-    Detects pricings and quantities in the input
+    Detects pricing and quantities in the input
 
     ## Output Attributes
 
-    `pricings: list[Label]`
-        A list of `Label` objects, with detected pricings and their data
+    `pricing: list[Label]`
+        A list of `Label` objects, with detected pricing and their data
 
     ## Example
 
     >>> pipeline = oneai.Pipeline(steps=[
-    ...     oneai.skills.Pricings()
+    ...     oneai.skills.Pricing()
     ... ])
     >>> output = pipeline.run('it costs 10 USD per barrel')
-    >>> output.pricings
+    >>> output.pricing
     [oneai.Label(name=pricing, data={'amount': 10.0, 'currency': 'USD', 'unit': 'barrel'}, span=[9, 26], value=10 USD per barrel)]
     """
 
